#include <stdio.h>
#include <string.h>
#include "environment.h"
#include "device/device_config.h"

#include <network.h>

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include <lwip/sockets.h>

#include "esp_system.h"
#include "esp_err.h"
#include "esp_event_loop.h"
#include "esp_event.h"
#include "esp_attr.h"
#include "esp_log.h"
#include "esp_eth.h"

#include "rom/ets_sys.h"
#include "rom/gpio.h"

#include "soc/dport_reg.h"
#include "soc/io_mux_reg.h"
#include "soc/rtc_cntl_reg.h"
#include "soc/gpio_reg.h"
#include "soc/gpio_sig_map.h"

#include "tcpip_adapter.h"
#include "nvs_flash.h"
#include "driver/gpio.h"

#include "soc/emac_ex_reg.h"
#include "driver/periph_ctrl.h"


#include "eth_phy/phy_lan8720.h"

#define DEFAULT_ETHERNET_PHY_CONFIG phy_lan8720_default_ethernet_config

static const char *TAG = "ethernet";

#define PIN_PHY_POWER GPIO_NUM_18
#define PIN_SMI_MDC   GPIO_NUM_16
#define PIN_SMI_MDIO  GPIO_NUM_17

static void phy_device_power_enable_via_gpio(bool enable)
{
    assert(DEFAULT_ETHERNET_PHY_CONFIG.phy_power_enable);

    if (!enable) {
        /* Do the PHY-specific power_enable(false) function before powering down */
        DEFAULT_ETHERNET_PHY_CONFIG.phy_power_enable(false);
    }

    gpio_pad_select_gpio(PIN_PHY_POWER);
    gpio_set_direction(PIN_PHY_POWER, GPIO_MODE_OUTPUT);
    if(enable == true) {
        gpio_set_level(PIN_PHY_POWER, 1);
        ESP_LOGD(TAG, "phy_device_power_enable(TRUE)");
    } else {
        gpio_set_level(PIN_PHY_POWER, 0);
        ESP_LOGD(TAG, "phy_device_power_enable(FALSE)");
    }

    // Allow the power up/down to take effect, min 300us
    vTaskDelay(1);

    if (enable) {
        /* Run the PHY-specific power on operations now the PHY has power */
        DEFAULT_ETHERNET_PHY_CONFIG.phy_power_enable(true);
    }
}

static void eth_gpio_config_rmii(void)
{
    // RMII data pins are fixed:
    // TXD0 = GPIO19
    // TXD1 = GPIO22
    // TX_EN = GPIO21
    // RXD0 = GPIO25
    // RXD1 = GPIO26
    // CLK == GPIO0
    phy_rmii_configure_data_interface_pins();
    // MDC and MDIO can be configured
    phy_rmii_smi_configure_pins(PIN_SMI_MDC, PIN_SMI_MDIO);
}

void enable_ethernet_lan8720()
{
    initialize_network();

    #ifdef ETH_STATIC_IP
    #ifdef ETH_IP
    tcpip_adapter_dhcpc_stop(TCPIP_ADAPTER_IF_ETH);
    tcpip_adapter_ip_info_t ipInfo;
    inet_pton(AF_INET, ETH_IP, &ipInfo.ip);
    inet_pton(AF_INET, ETH_GATEWAY, &ipInfo.gw);
    inet_pton(AF_INET, ETH_NETMASK, &ipInfo.netmask);
    tcpip_adapter_set_ip_info(TCPIP_ADAPTER_IF_ETH, &ipInfo);
    #endif
    #endif

    eth_config_t config = DEFAULT_ETHERNET_PHY_CONFIG;
    config.phy_addr = PHY1;
    config.gpio_config = eth_gpio_config_rmii;
    config.tcpip_input = tcpip_adapter_eth_input;
    config.clock_mode = ETH_CLOCK_GPIO0_IN;
    config.phy_power_enable = phy_device_power_enable_via_gpio;

    esp_err_t ret = ESP_OK;
    ret = esp_eth_init(&config);
    if (ret == ESP_OK) {
        esp_eth_enable();
    }
    else {
        ESP_LOGE(TAG, "Ethernet could not be initialized (%i)", ret);
    }
}