#include "qthing.h"
#include "network.h"
#include "io.h"
#include "environment.h"
#include "device/device_config.h"
#include "event.h"
#include "mqtt.h"

#include "string.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"
#include <lwip/sockets.h>
#include "esp_wifi.h"
#include "esp_log.h"
#include "esp_event_loop.h"
#include "nvs_flash.h"

void enable_wlan() {
    log_oled("WLAN initializing");
    update_wlan_connection_status(connecting);

    // Initialize NVS
    esp_err_t ret = nvs_flash_init();
    if (ret == ESP_ERR_NVS_NO_FREE_PAGES || ret == ESP_ERR_NVS_NEW_VERSION_FOUND) {
        ESP_ERROR_CHECK(nvs_flash_erase());
        ret = nvs_flash_init();
    }
    ESP_ERROR_CHECK( ret );

    initialize_network();

    #ifdef WLAN_STATIC_IP
    tcpip_adapter_dhcpc_stop(TCPIP_ADAPTER_IF_STA);
    tcpip_adapter_ip_info_t ipInfo;
    inet_pton(AF_INET, WLAN_IP, &ipInfo.ip);
    inet_pton(AF_INET, WLAN_GATEWAY, &ipInfo.gw);
    inet_pton(AF_INET, WLAN_NETMASK, &ipInfo.netmask);
    tcpip_adapter_set_ip_info(TCPIP_ADAPTER_IF_STA, &ipInfo);
    #endif

    wifi_init_config_t cfg = WIFI_INIT_CONFIG_DEFAULT();
    ESP_ERROR_CHECK(esp_wifi_init(&cfg));
    wifi_config_t wifi_config;
    memset(&wifi_config, sizeof(wifi_config_t), 1);
    wifi_config.sta = wifi_sta_config_t {
        WLAN_SSID,
        WLAN_PASSPHRASE,
        WIFI_ALL_CHANNEL_SCAN,
    };

    ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_STA));
    ESP_ERROR_CHECK(esp_wifi_set_config(ESP_IF_WIFI_STA, &wifi_config));
    ESP_ERROR_CHECK(esp_wifi_start());
}
